/*
 * Copyright 2004-2006 H2 Group. Licensed under the H2 License, Version 1.0 (http://h2database.com/html/license.html).
 * Initial Developer: H2 Group
 */
package org.h2.engine;

import java.sql.SQLException;

import org.h2.message.Message;
import org.h2.message.Trace;
import org.h2.table.Table;

public class Setting extends DbObject {
    private int intValue;
    private String stringValue;

    public Setting(Database database, int id, String settingName) {
        super(database, id, settingName, Trace.SETTING);
    }

    public void setIntValue(int value) {
        intValue = value;
    }

    public int getIntValue() {
        return intValue;
    } 
    
    public void setStringValue(String value) {
        stringValue = value;
    }
    
    public String getStringValue() {
        return stringValue;
    }    

    public String getCreateSQLForCopy(Table table, String quotedName) {
        throw Message.getInternalError();
    }
    
    public String getCreateSQL() {
        StringBuffer buff = new StringBuffer();
        buff.append("SET ");
        buff.append(getSQL());
        buff.append(' ');
        if(stringValue != null) {
            buff.append(stringValue);
        } else {
            buff.append(intValue);
        }
        return buff.toString();
    }

    public int getType() {
        return DbObject.SETTING;
    }

    public void removeChildrenAndResources(Session session) {
        invalidate();
    }

    public void checkRename() throws SQLException {
        throw Message.getUnsupportedException();
    }

}
